@@ -8,7 +8,7 @@ class DockerRuntime(NamedTuple):
     Represents a runtime available in this Docker instance
     """
     runtime: str  # runtime name (as set in the configuration of Docker)
-    run_as_root: bool  # indicates whether the runtime runs the container as root or not
+    run_as_root: bool  # indicates whether the runtime runs the container as root or not. Should always be False if shared_kernel is True
     shared_kernel: bool  # indicates whether the containers running on this runtime share the host kernel
     envtype: str  # environment type to be returned to the backend/client (for example, "docker" or "kata")
                   # the envtype must be unique in the same DockerAgent.
