@@ -25,7 +25,7 @@ WaitingJob = namedtuple('WaitingJob', ['priority', 'time_received', 'client_addr
 
 RunningJob = namedtuple('RunningJob', ['agent_addr', 'client_addr', 'msg', 'time_started'])
 EnvironmentInfo = namedtuple('EnvironmentInfo', ['last_id', 'created_last', 'agents', 'type'])
-AgentInfo = namedtuple('AgentInfo', ['name', 'environments'])  # environments is a list of tuple (type, environment)
+AgentInfo = namedtuple('AgentInfo', ['name', 'environments', 'ssh_allowed'])  # environments is a list of tuple (type, environment)
 
 class Backend(object):
     """
@@ -138,7 +138,7 @@ class Backend(object):
         self._logger.info("Adding a new job %s %s to the queue", client_addr, message.job_id)
         job = WaitingJob(message.priority, time.time(), client_addr, message.job_id, message)
         self._waiting_jobs[message.job_id] = job
-        self._waiting_jobs_pq.put((message.environment_type, message.environment), job)
+        self._waiting_jobs_pq.put((message.environment_type, message.environment, self._get_ssh_allowed(message)), job)
 
         await self.update_queue()
 
@@ -147,15 +147,18 @@ class Backend(object):
         # Check if the job is not in the waiting list
         if message.job_id in self._waiting_jobs:
             # Erase the job in waiting list
-            del self._waiting_jobs[message.job_id]
+            waiting_job = self._waiting_jobs.pop(message.job_id)
+            previous_state = waiting_job.msg.inputdata.get("@state", "")
 
             # Do not forget to send a JobDone
             await ZMQUtils.send_with_addr(self._client_socket, client_addr, BackendJobDone(message.job_id, ("killed", "You killed the job"),
-                                                                                           0.0, {}, {}, {}, "", None, "", ""))
+                                                                                           0.0, {}, {}, {}, previous_state, None, "", ""))
         # If the job is running, transmit the info to the agent
         elif message.job_id in self._job_running:
-            agent_addr = self._job_running[message.job_id].agent_addr
-            await ZMQUtils.send_with_addr(self._agent_socket, agent_addr, BackendKillJob(message.job_id))
+            running_job = self._job_running[message.job_id]
+            agent_addr = running_job.agent_addr
+            previous_state = running_job.msg.inputdata.get("@state", "")
+            await ZMQUtils.send_with_addr(self._agent_socket, agent_addr, BackendKillJob(message.job_id, previous_state))
         else:
             self._logger.warning("Client %s attempted to kill unknown job %s", str(client_addr), str(message.job_id))
 
@@ -193,7 +196,12 @@ class Backend(object):
                 job = None
                 while job is None:
                     # keep the object, do not unzip it directly! It's sometimes modified when a job is killed.
-                    job = self._waiting_jobs_pq.get(self._registered_agents[agent_addr].environments)
+
+                    topics = [(*env, False) for env in self._registered_agents[agent_addr].environments]
+                    if self._registered_agents[agent_addr].ssh_allowed:
+                        topics += [(*env, True) for env in self._registered_agents[agent_addr].environments]
+
+                    job = self._waiting_jobs_pq.get(topics)
                     priority, insert_time, client_addr, job_id, job_msg = job
 
                     # Ensure the job has not been removed (killed)
@@ -230,7 +238,7 @@ class Backend(object):
 
         self._registered_agents[agent_addr] = AgentInfo(message.friendly_name,
                                                         [(etype, env) for etype, envs in
-                                                         message.available_environments.items() for env in envs])
+                                                         message.available_environments.items() for env in envs], message.ssh_allowed)
         self._available_agents.extend([agent_addr for _ in range(0, message.available_job_slots)])
         self._ping_count[agent_addr] = 0
 
@@ -399,4 +407,11 @@ class Backend(object):
         try:
             return int(job_info.environment_parameters["limits"]["time"])
         except:
-            return -1 # unknown
\ No newline at end of file
+            return -1 # unknown
+
+    def _get_ssh_allowed(self, job_info: ClientNewJob):
+        """
+            Returns if the job requires that the agent allows ssh
+            For this to work, ["ssh_allowed"] must be a parameter of the environment.
+        """
+        return job_info.environment_parameters.get("ssh_allowed", False)
