@@ -1,10 +1,12 @@
 # coding=utf-8
+from __future__ import annotations
+
 import mimetypes
 import os
 import shutil
 import zipstream
 
-from inginious.common.filesystems.provider import FileSystemProvider, NotFoundException
+from inginious.common.filesystems import FileSystemProvider
 
 
 class LocalFSProvider(FileSystemProvider):
@@ -13,7 +15,7 @@ class LocalFSProvider(FileSystemProvider):
     """
     @classmethod
     def get_needed_args(cls):
-        """ Returns a list of arguments needed to create a FileSystemProvider. In the form 
+        """ Returns a list of arguments needed to create a FileSystemProvider. In the form
             {
                 "arg1": (int, False, "description1"),
                 "arg2: (str, True, "description2")
@@ -31,14 +33,11 @@ class LocalFSProvider(FileSystemProvider):
         """ Given the args from get_needed_args, creates the FileSystemProvider """
         return LocalFSProvider(location)
 
-    def __init__(self, prefix):
-        super().__init__(prefix)
-
-    def from_subfolder(self, subfolder):
+    def from_subfolder(self, subfolder: str) -> LocalFSProvider:
         self._checkpath(subfolder)
-        return LocalFSProvider(self.prefix + "/" + subfolder)
+        return LocalFSProvider(self.prefix + subfolder)
 
-    def exists(self, path=None):
+    def exists(self, path: str=None) -> bool:
         if path is None:
             path = self.prefix
         else:
@@ -59,21 +58,21 @@ class LocalFSProvider(FileSystemProvider):
             content = content.encode("utf-8")
         open(fullpath, 'wb').write(content)
 
-    def get_fd(self, filepath, timestamp=None):
+    def get_fd(self, filepath: str, timestamp=None):
         self._checkpath(filepath)
         return open(os.path.join(self.prefix, filepath), 'rb')
 
     def get(self, filepath, timestamp=None):
         return self.get_fd(filepath, timestamp).read()
 
-    def list(self, folders=True, files=True, recursive=False):
+    def list(self, folders: bool=True, files: bool=True, recursive: bool=False) -> list:
         if recursive:
             output = []
-            for root, subdirs, files in os.walk(self.prefix):
+            for root, subdirs, listed_files in os.walk(self.prefix):
                 if folders:
                     output += [root+"/"+d for d in subdirs]
                 if files:
-                    output += [root+"/"+f for f in files]
+                    output += [root+"/"+f for f in listed_files]
             output = [os.path.relpath(f, self.prefix) for f in output]
         else:
             if files and folders:
@@ -88,7 +87,7 @@ class LocalFSProvider(FileSystemProvider):
         isdir = lambda x: '/' if os.path.isdir(os.path.join(self.prefix, x)) else ''
         return [f+isdir(f) for f in output]
 
-    def delete(self, filepath=None):
+    def delete(self, filepath: str=None):
         if filepath is None:
             filepath = self.prefix
         else:
@@ -105,7 +104,7 @@ class LocalFSProvider(FileSystemProvider):
         try:
             return os.stat(os.path.join(self.prefix, filepath)).st_mtime
         except:
-            raise NotFoundException()
+            raise FileNotFoundError()
 
     def move(self, src, dest):
         self._checkpath(src)
@@ -139,11 +138,14 @@ class LocalFSProvider(FileSystemProvider):
             if not os.path.isdir(dest):
                 os.makedirs(dest)
             files = os.listdir(src)
-            for f in files:
-                self._recursive_overwrite(os.path.join(src, f),
-                                          os.path.join(dest, f))
+            for file in files:
+                self._recursive_overwrite(os.path.join(src, file),
+                                          os.path.join(dest, file))
         else:
-            shutil.copyfile(src, dest, follow_symlinks=False)
+            dirname = os.path.dirname(dest)
+            if not os.path.exists(dirname):
+                os.makedirs(dirname)
+            shutil.copy(src, dest, follow_symlinks=False)
 
     def distribute(self, filepath, allow_folders=True):
         self._checkpath(filepath)
