@@ -12,14 +12,16 @@ import urllib.parse
 import urllib.request
 import random
 import time
+import flask
 
-import web
+from flask import redirect, Response
+from werkzeug.exceptions import NotFound, HTTPException
 from bson.objectid import ObjectId
-from collections import OrderedDict
 from pymongo import ReturnDocument
 
-from inginious.common import exceptions
-from inginious.frontend.pages.utils import INGIniousPage
+from inginious.common.exceptions import TaskNotFoundException, CourseNotFoundException
+from inginious.frontend.pages.course import handle_course_unavailable
+from inginious.frontend.pages.utils import INGIniousPage, INGIniousAuthPage
 
 
 class BaseTaskPage(object):
@@ -37,107 +39,70 @@ class BaseTaskPage(object):
         self.webterm_link = self.cp.webterm_link
         self.plugin_manager = self.cp.plugin_manager
 
-    def set_selected_submission(self, course, task, submissionid):
-        """ Set submission whose id is `submissionid` to selected grading submission for the given course/task.
-            Returns a boolean indicating whether the operation was successful or not.
-        """
+    def preview_allowed(self, courseid, taskid):
+        try:
+            course = self.course_factory.get_course(courseid)
+        except CourseNotFoundException as ex:
+            raise NotFound(description=str(ex))
+        return course.get_accessibility().is_open() and course.allow_preview()
 
-        submission = self.submission_manager.get_submission(submissionid)
-
-        # Do not continue if submission does not exist or is not owned by current user
-        if not submission:
-            return False
-
-        # Check if the submission if from this task/course!
-        if submission["taskid"] != task.get_id() or submission["courseid"] != course.get_id():
-            return False
-
-        is_staff = self.user_manager.has_staff_rights_on_course(course, self.user_manager.session_username())
-
-        # Do not enable submission selection after deadline
-        if not task.get_accessible_time().is_open() and not is_staff:
-            return False
-
-        # Only allow to set submission if the student must choose their best submission themselves
-        if task.get_evaluate() != 'student' and not is_staff:
-            return False
-
-        # Check if task is done per group/team
-        if task.is_group_task() and not is_staff:
-            group = self.database.aggregations.find_one(
-                {"courseid": task.get_course_id(), "groups.students": self.user_manager.session_username()},
-                {"groups": {"$elemMatch": {"students": self.user_manager.session_username()}}})
-            students = group["groups"][0]["students"]
-        else:
-            students = [self.user_manager.session_username()]
-
-        # Check if group/team is the same
-        if students == submission["username"]:
-            self.database.user_tasks.update_many(
-                {"courseid": task.get_course_id(), "taskid": task.get_id(), "username": {"$in": students}},
-                {"$set": {"submissionid": submission['_id'],
-                          "grade": submission['grade'],
-                          "succeeded": submission["result"] == "success"}})
-            return True
-        else:
-            return False
-
-    def GET(self, courseid, taskid, isLTI):
+    def GET(self, courseid, taskid, is_LTI):
         """ GET request """
         username = self.user_manager.session_username()
 
         # Fetch the course
         try:
             course = self.course_factory.get_course(courseid)
-        except exceptions.CourseNotFoundException as ex:
-            raise web.notfound(str(ex))
+        except CourseNotFoundException as ex:
+            raise NotFound(description=str(ex))
 
-        if isLTI and not self.user_manager.course_is_user_registered(course):
+        if is_LTI and not self.user_manager.course_is_user_registered(course):
             self.user_manager.course_register_user(course, force=True)
 
-        if not self.user_manager.course_is_open_to_user(course, username, isLTI):
-            return self.template_helper.get_renderer().course_unavailable()
+        if not self.user_manager.course_is_open_to_user(course, username, is_LTI):
+            return handle_course_unavailable(self.cp.app.get_homepath(), self.template_helper, self.user_manager, course)
+
+        is_staff = self.user_manager.has_staff_rights_on_course(course, username)
 
-        # Fetch the task
         try:
-            tasks = OrderedDict((tid, t) for tid, t in course.get_tasks().items() if self.user_manager.task_is_visible_by_user(t, username, isLTI))
-            task = tasks[taskid]
-        except KeyError:
-            raise web.notfound()
+            task = course.get_task(taskid)
+            if not self.user_manager.task_is_visible_by_user(task, username, is_LTI):
+                return self.template_helper.render("task_unavailable.html")
+        except TaskNotFoundException:
+            raise NotFound()
 
-        if not self.user_manager.task_is_visible_by_user(task, username, isLTI):
-            return self.template_helper.get_renderer().task_unavailable()
-
-        # Compute previous and next taskid
-        keys = list(tasks.keys())
-        index = keys.index(taskid)
-        previous_taskid = keys[index - 1] if index > 0 else None
-        next_taskid = keys[index + 1] if index < len(keys) - 1 else None
+        user_task_list = course.get_task_dispenser().get_user_task_list([username])[username]
+        if taskid not in user_task_list:
+            previous_taskid = None
+            next_taskid = None
+        else:
+            # Compute previous and next taskid
+            index = user_task_list.index(taskid)
+            previous_taskid = user_task_list[index - 1] if index > 0 else None
+            next_taskid = user_task_list[index + 1] if index < len(user_task_list) - 1 else None
 
         self.user_manager.user_saw_task(username, courseid, taskid)
 
         is_staff = self.user_manager.has_staff_rights_on_course(course, username)
 
-        userinput = web.input()
+        userinput = flask.request.args
         if "submissionid" in userinput and "questionid" in userinput:
             # Download a previously submitted file
             submission = self.submission_manager.get_submission(userinput["submissionid"], user_check=not is_staff)
             if submission is None:
-                raise web.notfound()
+                raise self.cp.app.notfound(message=_("Submission doesn't exist."))
             sinput = self.submission_manager.get_input_from_submission(submission, True)
             if userinput["questionid"] not in sinput:
-                raise web.notfound()
+                raise NotFound()
 
             if isinstance(sinput[userinput["questionid"]], dict):
                 # File uploaded previously
                 mimetypes.init()
                 mime_type = mimetypes.guess_type(urllib.request.pathname2url(sinput[userinput["questionid"]]['filename']))
-                web.header('Content-Type', mime_type[0])
-                return sinput[userinput["questionid"]]['value']
+                return Response(response=sinput[userinput["questionid"]]['value'], content_type=mime_type[0])
             else:
                 # Other file, download it as text
-                web.header('Content-Type', 'text/plain')
-                return sinput[userinput["questionid"]]
+                return Response(response=sinput[userinput["questionid"]], content_type='text/plain')
         else:
             # Generate random inputs and save it into db
             random.seed(str(username if username is not None else "") + taskid + courseid + str(
@@ -161,19 +126,32 @@ class BaseTaskPage(object):
 
             students = [self.user_manager.session_username()]
             if task.is_group_task() and not self.user_manager.has_admin_rights_on_course(course, username):
-                group = self.database.aggregations.find_one(
-                    {"courseid": task.get_course_id(), "groups.students": self.user_manager.session_username()},
-                    {"groups": {"$elemMatch": {"students": self.user_manager.session_username()}}})
-                if group is not None and len(group["groups"]) > 0:
-                    students = group["groups"][0]["students"]
+                group = self.database.groups.find_one({"courseid": task.get_course_id(),
+                                                     "students": self.user_manager.session_username()})
+                if group is not None:
+                    students = group["students"]
                 # we don't care for the other case, as the student won't be able to submit.
 
             submissions = self.submission_manager.get_user_submissions(task) if self.user_manager.session_logged_in() else []
-            user_info = self.database.users.find_one({"username": username})
+            user_info = self.user_manager.get_user_info(username)
+
+            # Visible tags
+            course_tags = course.get_tags()
+            task_categories = task.get_categories()
+            visible_tags = [course_tags[category] for category in task_categories if
+                            course_tags[category].is_visible_for_student() or self.user_manager.has_staff_rights_on_course(course)]
+
+            # Problem dict
+            pdict = {problem.get_id(): problem.get_type() for problem in task.get_problems()}
+            is_input_list = {problem.get_id():  1 if problem.input_type() == list else 0 for problem in task.get_problems()}
 
             # Display the task itself
-            return self.template_helper.get_renderer().task(user_info, course, task, submissions,
-                                                            students, eval_submission, user_task, previous_taskid, next_taskid, self.webterm_link, random_input_list)
+            return self.template_helper.render("task.html", user_info=user_info, course=course, task=task,
+                                               submissions=submissions, students=students,
+                                               eval_submission=eval_submission, user_task=user_task,
+                                               previous_taskid=previous_taskid, next_taskid=next_taskid,
+                                               webterm_link=self.webterm_link, input_random_list=random_input_list,
+                                               visible_tags=visible_tags, pdict=pdict, is_input_list=is_input_list)
 
     def POST(self, courseid, taskid, isLTI):
         """ POST a new submission """
@@ -181,22 +159,22 @@ class BaseTaskPage(object):
 
         course = self.course_factory.get_course(courseid)
         if not self.user_manager.course_is_open_to_user(course, username, isLTI):
-            return self.template_helper.get_renderer().course_unavailable()
-
-        task = course.get_task(taskid)
-        if not self.user_manager.task_is_visible_by_user(task, username, isLTI):
-            return self.template_helper.get_renderer().task_unavailable()
-
-        self.user_manager.user_saw_task(username, courseid, taskid)
+            return handle_course_unavailable(self.cp.app.get_homepath(), self.template_helper, self.user_manager, course)
 
         is_staff = self.user_manager.has_staff_rights_on_course(course, username)
         is_admin = self.user_manager.has_admin_rights_on_course(course, username)
 
-        userinput = web.input()
+        task = course.get_task(taskid)
+        if not self.user_manager.task_is_visible_by_user(task, username, isLTI):
+            return self.template_helper.render("task_unavailable.html")
+
+        self.user_manager.user_saw_task(username, courseid, taskid)
+
+        userinput = flask.request.form
         if "@action" in userinput and userinput["@action"] == "submit":
             # Verify rights
             if not self.user_manager.task_can_user_submit(task, username, isLTI):
-                return json.dumps({"status": "error", "text": _("You are not allowed to submit for this task.")})
+                return json.dumps({"status": "error", "title": _("Error"), "text": _("You are not allowed to submit for this task.")})
 
             # Retrieve input random and check still valid
             random_input = self.database.user_tasks.find_one({"courseid": task.get_course_id(), "taskid": task.get_id(), "username": username}, { "random": 1 })
@@ -204,19 +182,28 @@ class BaseTaskPage(object):
             for i in range(0, len(random_input)):
                 s = "@random_" + str(i)
                 if s not in userinput or float(userinput[s]) != random_input[i]:
-                    return json.dumps({"status": "error", "text": _("Your task has been regenerated. This current task is outdated.")})
+                    return json.dumps({"status": "error", "title": _("Error"), "text": _("Your task has been regenerated. This current task is outdated.")})
 
-            # Reparse user input with array for multiple choices
-            init_var = {
-                problem.get_id(): problem.input_type()()
-                for problem in task.get_problems() if problem.input_type() in [dict, list]
-            }
-            userinput = task.adapt_input_for_backend(web.input(**init_var))
+            # Reparse user input with array for multiple choices and files
+            userinput = flask.request.form.copy()
+            for problem in task.get_problems():
+                pid = problem.get_id()
+                if problem.input_type() == list:
+                    userinput[pid] = flask.request.form.getlist(pid)
+                elif problem.input_type() == dict:
+                    userinput[pid] = flask.request.files.get(pid)
+                else:
+                    userinput[pid] = flask.request.form.get(pid)
+
+            userinput = task.adapt_input_for_backend(userinput)
 
             if not task.input_is_consistent(userinput, self.default_allowed_file_extensions, self.default_max_file_size):
-                web.header('Content-Type', 'application/json')
-                return json.dumps({"status": "error", "text": _("Please answer to all the questions and verify the extensions of the files "
-                                                              "you want to upload. Your responses were not tested.")})
+                return Response(content_type='application/json',
+                                response=json.dumps({
+                                    "status": "error",  "title": _("Error"),
+                                    "text": _("Please answer to all the questions and verify the extensions of the files "
+                                              "you want to upload. Your responses were not tested.")
+                                }))
             del userinput['@action']
 
             # Get debug info if the current user is an admin
@@ -229,19 +216,22 @@ class BaseTaskPage(object):
             # Start the submission
             try:
                 submissionid, oldsubids = self.submission_manager.add_job(task, userinput, debug)
-                web.header('Content-Type', 'application/json')
-                return json.dumps({"status": "ok", "submissionid": str(submissionid), "remove": oldsubids, "text": _("<b>Your submission has been sent...</b>")})
+                return Response(content_type='application/json', response=json.dumps({
+                    "status": "ok", "submissionid": str(submissionid), "remove": oldsubids,
+                    "text": _("<b>Your submission has been sent...</b>")
+                }))
             except Exception as ex:
-                web.header('Content-Type', 'application/json')
-                return json.dumps({"status": "error", "text": str(ex)})
+                return Response(content_type='application/json', response=json.dumps({
+                    "status": "error", "title": _("Error"), "text": str(ex)
+                }))
 
         elif "@action" in userinput and userinput["@action"] == "check" and "submissionid" in userinput:
             result = self.submission_manager.get_submission(userinput['submissionid'], user_check=not is_staff)
             if result is None:
-                web.header('Content-Type', 'application/json')
-                return json.dumps({'status': "error", "text": _("Internal error")})
+                return Response(content_type='application/json', response=json.dumps({
+                    'status': "error",  "title": _("Error"), "text": _("Internal error")
+                }))
             elif self.submission_manager.is_done(result, user_check=not is_staff):
-                web.header('Content-Type', 'application/json')
                 result = self.submission_manager.get_input_from_submission(result)
                 result = self.submission_manager.get_feedback_from_submission(result, show_everything=is_staff)
 
@@ -255,38 +245,34 @@ class BaseTaskPage(object):
                 default_submissionid = user_task.get('submissionid', None)
                 if default_submissionid is None:
                     # This should never happen, as user_manager.update_user_stats is called whenever a submission is done.
-                    return json.dumps({'status': "error", "text": _("Internal error")})
+                    return Response(content_type='application/json', response=json.dumps({
+                        'status': "error",  "title": _("Error"), "text": _("Internal error")
+                    }))
 
-                return self.submission_to_json(task, result, is_admin, False, default_submissionid == result['_id'], tags=course.get_tags())
+                return Response(content_type='application/json', response=self.submission_to_json(
+                    task, result, is_admin, False, default_submissionid == result['_id'], tags=course.get_tags()
+                ))
             else:
-                web.header('Content-Type', 'application/json')
-                return self.submission_to_json(task, result, is_admin, False, tags=course.get_tags())
+                return Response(content_type='application/json', response=self.submission_to_json(
+                    task, result, is_admin, False, tags=course.get_tags()
+                ))
 
         elif "@action" in userinput and userinput["@action"] == "load_submission_input" and "submissionid" in userinput:
             submission = self.submission_manager.get_submission(userinput["submissionid"], user_check=not is_staff)
             submission = self.submission_manager.get_input_from_submission(submission)
             submission = self.submission_manager.get_feedback_from_submission(submission, show_everything=is_staff)
             if not submission:
-                raise web.notfound()
-            web.header('Content-Type', 'application/json')
+                raise NotFound(description=_("Submission doesn't exist."))
 
-            return self.submission_to_json(task, submission, is_admin, True, tags=course.get_tags())
+            return Response(content_type='application/json', response=self.submission_to_json(
+                task, submission, is_admin, True, tags=course.get_tags()
+            ))
 
         elif "@action" in userinput and userinput["@action"] == "kill" and "submissionid" in userinput:
             self.submission_manager.kill_running_submission(userinput["submissionid"])  # ignore return value
-            web.header('Content-Type', 'application/json')
-            return json.dumps({'status': 'done'})
-        elif "@action" in userinput and userinput["@action"] == "set_submission" and "submissionid" in userinput:
-            web.header('Content-Type', 'application/json')
-            if task.get_evaluate() != 'student':
-                return json.dumps({'status': "error"})
-
-            if self.set_selected_submission(course, task, userinput["submissionid"]):
-                return json.dumps({'status': 'done'})
-            else:
-                return json.dumps({'status': 'error'})
+            return Response(content_type='application/json', response=json.dumps({'status': 'done'}))
         else:
-            raise web.notfound()
+            raise NotFound()
 
     def submission_to_json(self, task, data, debug, reloading=False, replace=False, tags=None):
         """ Converts a submission to json (keeps only needed fields) """
@@ -295,9 +281,14 @@ class BaseTaskPage(object):
             tags = []
 
         if "ssh_host" in data:
-            return json.dumps({'status': "waiting", 'text': "<b>SSH server active</b>",
-                               'ssh_host': data["ssh_host"], 'ssh_port': data["ssh_port"],
-                               'ssh_password': data["ssh_password"]})
+            return json.dumps({
+                'status': "waiting",
+                'text': "<b>SSH server active</b>",
+                'ssh_host': data["ssh_host"],
+                'ssh_port': data["ssh_port"],
+                'ssh_user': data["ssh_user"],
+                'ssh_password': data["ssh_password"]
+            })
 
         # Here we are waiting. Let's send some useful information.
         waiting_data = self.submission_manager.get_job_queue_info(data["jobid"]) if "jobid" in data else None
@@ -305,9 +296,9 @@ class BaseTaskPage(object):
             nb_tasks_before, approx_wait_time = waiting_data
             wait_time = round(approx_wait_time)
             if nb_tasks_before == -1 and wait_time <= 0:
-                text = _("<b>INGInious is currently grading your answers.<b/> (almost done)")
+                text = _("<b>INGInious is currently grading your answers.</b> (almost done)")
             elif nb_tasks_before == -1:
-                text = _("<b>INGInious is currently grading your answers.<b/> (Approx. wait time: {} seconds)").format(
+                text = _("<b>INGInious is currently grading your answers.</b> (Approx. wait time: {} seconds)").format(
                     wait_time)
             elif nb_tasks_before == 0:
                 text = _("<b>You are next in the waiting queue!</b>")
@@ -386,9 +377,9 @@ class BaseTaskPage(object):
             new_data = {}
             for key, value in data.items():
                 if isinstance(value, bytes) and len(value) > limit:
-                    new_data[key] = value[:limit] + _(" <text cut due to its length>").encode("utf-8")
+                    new_data[key] = value[:limit] + _(" <truncated>").encode("utf-8")
                 elif isinstance(value, str) and len(value) > limit:
-                    new_data[key] = value[:limit] + _(" <text cut due to its length>")
+                    new_data[key] = value[:limit] + _(" <truncated>")
                 elif isinstance(value, dict):
                     new_data[key] = self._cut_long_chains(value)
                 else:
@@ -409,7 +400,7 @@ class TaskPageStaticDownload(INGIniousPage):
         try:
             course = self.course_factory.get_course(courseid)
             if not self.user_manager.course_is_open_to_user(course):
-                return self.template_helper.get_renderer().course_unavailable()
+                return handle_course_unavailable(self.app.get_homepath(), self.template_helper, self.user_manager, course)
 
             path_norm = posixpath.normpath(urllib.parse.unquote(path))
 
@@ -419,30 +410,29 @@ class TaskPageStaticDownload(INGIniousPage):
 
                 task = course.get_task(taskid)
                 if not self.user_manager.task_is_visible_by_user(task):  # ignore LTI check here
-                    return self.template_helper.get_renderer().task_unavailable()
+                    return self.template_helper.render("task_unavailable.html")
 
                 public_folder = task.get_fs().from_subfolder("public")
             (method, mimetype_or_none, file_or_url) = public_folder.distribute(path_norm, False)
 
             if method == "local":
-                web.header('Content-Type', mimetype_or_none)
-                return file_or_url
+                return Response(response=file_or_url, content_type=mimetype_or_none)
             elif method == "url":
-                raise web.redirect(file_or_url)
+                return redirect(file_or_url)
             else:
-                raise web.notfound()
-        except web.HTTPError as error_or_redirect:
+                raise NotFound()
+        except TaskNotFoundException:
+            raise NotFound()
+        except HTTPException as error_or_redirect:
             raise error_or_redirect
-        except:
-            if web.config.debug:
-                raise
-            else:
-                raise web.notfound()
 
 
-class TaskPage(INGIniousPage):
-    def GET(self, courseid, taskid):
+class TaskPage(INGIniousAuthPage):
+    def GET_AUTH(self, courseid, taskid):
         return BaseTaskPage(self).GET(courseid, taskid, False)
 
-    def POST(self, courseid, taskid):
+    def POST_AUTH(self, courseid, taskid):
         return BaseTaskPage(self).POST(courseid, taskid, False)
+
+    def preview_allowed(self, courseid, taskid):
+        return BaseTaskPage(self).preview_allowed(courseid, taskid)
