@@ -190,10 +190,10 @@ class WebAppSubmissionManager:
             submission["input"] = self._gridfs.put(bson.BSON.encode(inputdata))
             submission["tests"] = {}  # Be sure tags are reinitialized
             submission["user_ip"] = flask.request.remote_addr
-            submissionid = self._database.submissions.insert(submission)
+            submissionid = self._database.submissions.insert_one(submission).inserted_id
 
         # Clean the submission document in db
-        self._database.submissions.update(
+        self._database.submissions.update_one(
             {"_id": submission["_id"]},
             {"$set": {"status": "waiting", "response_type": task.get_response_type()},
              "$unset": {"result": "", "grade": "", "text": "", "tests": "", "problems": "", "archive": "", "state": "",
@@ -207,9 +207,9 @@ class WebAppSubmissionManager:
                                      "Frontend - {}".format(submission["username"]), debug, ssh_callback)
 
 
-        self._database.submissions.update(
+        self._database.submissions.update_one(
             {"_id": submission["_id"], "status": "waiting"},
-            {"$set": {"jobid": jobid}}
+            {"$set": {"jobid": jobid,"last_replay": datetime.now()}}
         )
 
         if not copy:
@@ -284,11 +284,22 @@ class WebAppSubmissionManager:
         inputdata["@random"] = states["random"] if "random" in states else []
         inputdata["@state"] = states["state"] if "state" in states else ""
 
+        # Send LTI information to the client except "consumer_key"
+        lti_info = self._user_manager.session_lti_info()
+        if lti_info:
+            for key in lti_info:
+                if key == "consumer_key" or key.startswith("outcome"): # Skip "consumer_key" and "outcome*"
+                    continue
+                self._logger.debug("LTI data : %s, %s",key, lti_info[key])
+                # Add @lti_ prefix
+                key_str = "@lti_" + key
+                inputdata[key_str] = lti_info[key]
+
         self._plugin_manager.call_hook("new_submission", submission=obj, inputdata=inputdata)
 
         self._before_submission_insertion(task, inputdata, debug, obj)
         obj["input"] = self._gridfs.put(bson.BSON.encode(inputdata))
-        submissionid = self._database.submissions.insert(obj)
+        submissionid = self._database.submissions.insert_one(obj).inserted_id
         to_remove = self._after_submission_insertion(task, inputdata, debug, obj, submissionid)
 
         ssh_callback = lambda host, port, user, password: self._handle_ssh_callback(submissionid, host, port, user, password)
@@ -299,7 +310,7 @@ class WebAppSubmissionManager:
                                                               custom, state, archive, stdout, stderr, True)),
                                      "Frontend - {}".format(username), debug, ssh_callback)
 
-        self._database.submissions.update(
+        self._database.submissions.update_one(
             {"_id": submissionid, "status": "waiting"},
             {"$set": {"jobid": jobid}}
         )
@@ -392,10 +403,20 @@ class WebAppSubmissionManager:
                                                                     submission["response_type"],
                                                                     show_everything, translation).parse())
                 else:  # new-style submission
-                    submission["problems"][problem] = (
-                    submission["problems"][problem][0], ParsableText(submission["problems"][problem][1],
+
+                    try:
+                        submission["problems"][problem] = (
+                        submission["problems"][problem][0], ParsableText(submission["problems"][problem][1],
                                                                      submission["response_type"],
                                                                      show_everything, translation).parse())
+                    except TypeError:
+                        self._logger.error(
+                            "Something went wrong with provided feedback for submission %s", str(submission["_id"])
+                            )
+                        submission["problems"][problem] = (
+                            'crash', ParsableText(_("Feedback is badly formatted."),
+                                                                    submission["response_type"],
+                                                                    show_everything, translation).parse())
         return submission
 
     def is_running(self, submissionid, user_check=True):
@@ -689,7 +710,6 @@ def update_pending_jobs(database):
     """ Updates pending jobs status in the database """
 
     # Updates the submissions that are waiting with the status error, as the server restarted
-    database.submissions.update({'status': 'waiting'},
+    database.submissions.update_many({'status': 'waiting'},
                                 {"$unset": {'jobid': ""},
-                                 "$set": {'status': 'error', 'grade': 0.0, 'text': 'Internal error. Server restarted'}},
-                                multi=True)
+                                 "$set": {'status': 'error', 'grade': 0.0, 'text': 'Internal error. Server restarted'}})
